@@ -30,6 +30,8 @@
 #include <KSharedConfig>
 #include <KConfigGroup>
 
+extern KICONTHEMES_EXPORT void uintToHex(uint32_t colorData, QChar *buffer);
+
 class KIconLoader_UnitTest : public QObject
 {
     Q_OBJECT
@@ -210,7 +212,7 @@ private Q_SLOTS:
         QCOMPARE(pix.size(), QSize(16, 16));
         // And now set canReturnNull to true
         pix = iconLoader.loadIcon(QStringLiteral("this-icon-does-not-exist"), KIconLoader::Desktop, 16, KIconLoader::DefaultState,
-                                  QStringList(), 0, true);
+                                  QStringList(), nullptr, true);
         QVERIFY(pix.isNull());
         // Try getting the "unknown" icon again, to see if the above call didn't put a null icon into the cache...
         pix = iconLoader.loadIcon(QStringLiteral("this-icon-does-not-exist"), KIconLoader::Desktop, 16);
@@ -497,6 +499,26 @@ private Q_SLOTS:
         //that is the color we wrote in kdeglobals as text color?
         QCOMPARE(img.pixel(0,0), (uint)4294901760);
     }
+
+    void testUintToHex()
+    {
+        // HEX (ARGB format without the #): ff6496c8
+        QColor testColorNoAlpha(100, 150, 200);
+
+        // The ARGB string in which the composed hex value is stored.
+        QString argbHex(8, Qt::Uninitialized);
+
+        // Verify the ARGB hex (ff6496c8)
+        uintToHex(testColorNoAlpha.rgba(), argbHex.data());
+        QCOMPARE(argbHex, QString("ff6496c8"));
+
+        // HEX (ARGB format without the #): 7b6496c8
+        QColor testColorWithAlpha(100, 150, 200, 123);
+
+        // Test uintToHex to verify its ARGB output.
+        uintToHex(testColorWithAlpha.rgba(), argbHex.data());
+        QCOMPARE(argbHex, QString("7b6496c8"));
+    }
 };
 
 QTEST_MAIN(KIconLoader_UnitTest)
