@@ -72,14 +72,12 @@ void KIconEngine::paint(QPainter *painter, const QRect &rect, QIcon::Mode mode,
         return;
     }
 
-    Q_UNUSED(state)
-
-    qreal dpr = painter->device()->devicePixelRatio();
-
-    const int kstate = qIconModeToKIconState(mode);
-    const int iconSize = qMin(rect.width(), rect.height()) * dpr;
-    const QPixmap pix = mIconLoader.data()->loadIcon(mIconName, KIconLoader::Desktop, iconSize, kstate, mOverlays);
-    painter->drawPixmap(rect, pix);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
+    const qreal dpr = painter->device()->devicePixelRatioF();
+#else
+    const qreal dpr = painter->device()->devicePixelRatio();
+#endif
+    painter->drawPixmap(rect, pixmap(rect.size() * dpr, mode, state));
 }
 
 QPixmap KIconEngine::pixmap(const QSize &size, QIcon::Mode mode, QIcon::State state)
@@ -109,13 +107,16 @@ QPixmap KIconEngine::pixmap(const QSize &size, QIcon::Mode mode, QIcon::State st
     pix2.fill(QColor(0, 0, 0, 0));
 
     QPainter painter(&pix2);
-    painter.drawPixmap(QPoint(), pix);
+    painter.drawPixmap(QPoint((pix2.width() - pix.width()) / 2, (pix2.height() - pix.height()) / 2), pix);
 
     return pix2;
 }
 
 QString KIconEngine::iconName() const
 {
+    if (!mIconLoader || !mIconLoader->hasIcon(mIconName)) {
+        return QString();
+    }
     return mIconName;
 }
 
@@ -155,3 +156,15 @@ bool KIconEngine::write(QDataStream &out) const
     out << mIconName << mOverlays;
     return true;
 }
+
+void KIconEngine::virtual_hook(int id, void *data)
+{
+#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
+    if (id == QIconEngine::IsNullHook) {
+#else
+    if (id == 3) {
+#endif
+        *reinterpret_cast<bool*>(data) = !mIconLoader || !mIconLoader->hasIcon(mIconName);
+    }
+    QIconEngine::virtual_hook(id, data);
+}
